"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestingApolloClient = void 0;
const client_1 = require("@apollo/client");
// no-cache and network-only fetch policies cause an initial async event to be
// produced which is not properly caught inside of a react-testing act scope.
const invalidFetchPolicies = ['no-cache', 'network-only'];
// patch getCurrentQueryResult to protect against an initial observable event
class TestingApolloClient extends client_1.ApolloClient {
    constructor(options) {
        super(options);
        // this.getCurrentQueryResult = this.queryManager.getCurrentQueryResult.bind(
        //   this.queryManager,
        // );
        // this.queryManager.getCurrentQueryResult = this.getCurrentQueryResultSafe.bind(
        //   this,
        // );
    }
    getCurrentQueryResultSafe(observableQuery, optimistic) {
        const result = this.getCurrentQueryResult(observableQuery, optimistic);
        result.partial = isPartial(observableQuery, result);
        return result;
    }
}
exports.TestingApolloClient = TestingApolloClient;
// detect the conditions where we need to override partial to true
// see https://github.com/apollographql/apollo-client/blob/413b41211a232911824e7ede62e41e2910ae1a25/packages/apollo-client/src/core/QueryManager.ts#L1036-L1038
function isPartial(observableQuery, { data, partial }) {
    if (!observableQuery.getLastResult() &&
        invalidFetchPolicies.includes(observableQuery.options.fetchPolicy) &&
        data === undefined &&
        partial === false) {
        return true;
    }
    // if the conditions don't match we just return the original value
    return partial;
}
