"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphQL = void 0;
const client_1 = require("@apollo/client");
const client_2 = require("./client");
const links_1 = require("./links");
const operations_1 = require("./operations");
const utilities_1 = require("./utilities");
class GraphQL {
    constructor(mock, { possibleTypes = {}, cacheOptions = {} } = {}) {
        this.operations = new operations_1.Operations();
        this.pendingRequests = new Set();
        this.wrappers = [];
        this.mockLink = null;
        this.handleCreate = (request) => {
            this.pendingRequests.add(request);
        };
        this.handleResolve = (request) => {
            this.operations.push(request.operation);
            this.pendingRequests.delete(request);
        };
        const cache = new client_1.InMemoryCache({
            possibleTypes,
            ...cacheOptions,
        });
        this.mockLink = new links_1.MockLink(mock || defaultGraphQLMock);
        const link = client_1.ApolloLink.from([
            new links_1.InflightLink({
                onCreated: this.handleCreate,
                onResolved: this.handleResolve,
            }),
            this.mockLink,
        ]);
        this.client = new client_2.TestingApolloClient({
            link,
            cache,
        });
    }
    update(mock) {
        if (!this.mockLink) {
            return;
        }
        this.mockLink.updateMock(mock);
    }
    async resolveAll(options = {}) {
        const finalOperationName = utilities_1.operationNameFromFindOptions(options);
        await this.wrappers.reduce((perform, wrapper) => {
            return () => wrapper(perform);
        }, async () => {
            const allPendingRequests = Array.from(this.pendingRequests);
            const matchingRequests = finalOperationName
                ? allPendingRequests.filter(({ operation: { operationName } }) => operationName === finalOperationName)
                : allPendingRequests;
            await Promise.all(matchingRequests.map(({ resolve }) => resolve()));
        })();
    }
    wrap(wrapper) {
        this.wrappers.push(wrapper);
    }
}
exports.GraphQL = GraphQL;
function defaultGraphQLMock({ operationName }) {
    return new Error(`Can’t perform GraphQL operation '${operationName || ''}' because no mocks were set.`);
}
