"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InflightLink = void 0;
const client_1 = require("@apollo/client");
class InflightLink extends client_1.ApolloLink {
    constructor(options) {
        super();
        this.options = options;
    }
    request(operation, nextLink) {
        if (nextLink == null || !nextLink) {
            throw new Error('The memory link must not be a terminating link');
        }
        let resolver;
        const promise = new Promise(resolve => {
            resolver = resolve;
        });
        const request = {
            operation,
            resolve: () => {
                resolver();
                this.options.onResolved(request);
                return promise;
            },
        };
        this.options.onCreated(request);
        return new client_1.Observable(observer => {
            return nextLink(operation).subscribe({
                complete() {
                    const complete = observer.complete.bind(observer);
                    promise.then(complete).catch(complete);
                },
                next(result) {
                    const next = observer.next.bind(observer, result);
                    promise.then(next).catch(next);
                },
                error(error) {
                    const fail = observer.error.bind(observer, error);
                    promise.then(fail).catch(fail);
                },
            });
        });
    }
}
exports.InflightLink = InflightLink;
