"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockLink = void 0;
const client_1 = require("@apollo/client");
const graphql_1 = require("graphql");
class MockLink extends client_1.ApolloLink {
    constructor(mock) {
        super();
        this.mock = mock;
    }
    updateMock(mock) {
        this.mock = mock;
    }
    request(operation) {
        return new client_1.Observable(obs => {
            const { mock } = this;
            const { operationName = '' } = operation;
            let response = null;
            if (typeof mock === 'function') {
                response = mock(operation);
            }
            else {
                const mockForOperation = mock[operationName || ''];
                response =
                    typeof mockForOperation === 'function'
                        ? mockForOperation(operation)
                        : mockForOperation;
            }
            let result;
            if (response == null) {
                let message = `Can’t perform GraphQL operation '${operationName}' because no valid mocks were found`;
                if (typeof mock === 'object') {
                    const operationNames = Object.keys(mock);
                    // We will provide a more helpful message when it looks like they just provided data,
                    // not an object mapping names to fixtures.
                    const looksLikeDataNotFixtures = operationNames.every(name => name === name.toLowerCase());
                    message += looksLikeDataNotFixtures
                        ? ` (it looks like you tried to provide data directly to the mock GraphQL client. You need to provide your fixture on the key that matches its operation name. To fix this, simply change your code to read 'mockGraphQLClient({${operationName}: yourFixture}).'`
                        : ` (you provided an object that had mocks only for the following operations: ${Object.keys(mock).join(', ')}).`;
                }
                else {
                    message +=
                        ' (you provided a function that did not return a valid mock result)';
                }
                const error = new Error(message);
                result = error;
            }
            else if (response instanceof graphql_1.GraphQLError) {
                result = {
                    errors: [response],
                };
            }
            else if (response instanceof Error) {
                result = { errors: [new graphql_1.GraphQLError(response.message)] };
            }
            else {
                result = {
                    data: response,
                };
            }
            obs.next(result);
            obs.complete();
        });
    }
}
exports.MockLink = MockLink;
