"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toHavePerformedGraphQLOperation = void 0;
const jest_matcher_utils_1 = require("jest-matcher-utils");
const utilities_1 = require("../utilities");
const utilities_2 = require("./utilities");
function toHavePerformedGraphQLOperation(graphQL, document, variables) {
    utilities_2.assertIsGraphQL(graphQL, {
        expectation: 'toHavePerformedGraphQLOperation',
        isNot: this.isNot,
    });
    const foundByOperation = graphQL.operations.all({
        query: document,
    });
    const foundByVariables = variables == null
        ? foundByOperation
        : foundByOperation.filter(operation => Object.keys(variables).every(key => this.equals(variables[key], operation.variables[key])));
    const pass = foundByVariables.length > 0;
    const type = utilities_1.operationTypeFromDocument(document) || 'query';
    const name = utilities_1.operationNameFromDocument(document) || 'unnamed';
    const message = pass
        ? () => `${jest_matcher_utils_1.matcherHint('.not.toHavePerformedGraphQLOperation')}\n\n` +
            `Expected not to have performed GraphQL ${type}:\n  ${jest_matcher_utils_1.EXPECTED_COLOR(name)}\n${variables
                ? `With variables matching:\n  ${jest_matcher_utils_1.printExpected(variables)}\n`
                : ''}` +
            `But ${foundByVariables.length} matching ${foundByVariables.length === 1 ? 'operation was' : 'operations were'} found.\n`
        : () => `${`${jest_matcher_utils_1.matcherHint('.toHavePerformedGraphQLOperation')}\n\n` +
            `Expected to have performed GraphQL ${type}:\n  ${jest_matcher_utils_1.EXPECTED_COLOR(name)}\n${variables
                ? `With props matching:\n  ${jest_matcher_utils_1.printExpected(variables)}\n`
                : ''}`}${foundByVariables.length === 0
            ? `But no matching operations were found.\n`
            : `But the ${foundByVariables.length === 1
                ? 'found operation has'
                : 'found operations have'} the following variable differences:\n\n${diffs(foundByVariables, variables, this.expand)}`}`;
    return { pass, message };
}
exports.toHavePerformedGraphQLOperation = toHavePerformedGraphQLOperation;
function diffs(operations, variables, expand) {
    return operations.reduce((diffs, operation, index) => `${diffs}${index === 0 ? '' : '\n\n'}${normalizedDiff(operation, variables, {
        expand,
        showLegend: index === 0,
    })}`, '');
}
function normalizedDiff(operation, variables, { expand = false, showLegend = false }) {
    const result = utilities_2.diffVariablesForOperation(operation, variables, {
        expand,
    }) || '';
    return showLegend ? result : result.split('\n\n')[1];
}
